class CalipelFormBuilder < ActionView::Helpers::FormBuilder

  def field_item(attribute, &block)
    @template.content_tag(:div, class: "control-group #{'error' if errors_present?(attribute)}") do
      input = @template.content_tag(:div, class: "controls") do
        yield + errors_on(attribute)
      end
      "#{label_item attribute}\n#{input}".html_safe
    end
  end

  def label_item(attribute)
    label(attribute, class: "control-label")
  end

  def errors_on(attribute)
    if errors_present?(attribute)
      @template.content_tag("span",
                            @object.errors[attribute].to_sentence,
                            class: "help-inline")
    end
  end

  def fieldset_item(title, *args, &block)
    @template.content_tag("fieldset", *args) do
      @template.content_tag("legend", title) + @template.content_tag("ul", &block)
    end
  end

  def text_field_item(attribute, *args)
    form_input_item(:text_field, attribute, *args)
  end

  def date_field_item(attribute, *args)
    form_input_item(:date_field, attribute, *args)
  end

  def datetime_field_item(attribute, *args)
    form_input_item(:datetime_field, attribute, *args)
  end

  def text_area_item(attribute, *args)
    form_input_item(:text_area, attribute, *args)
  end

  def file_field_item(attribute, *args)
    form_input_item(:file_field, attribute, *args)
  end

  def checkboxes_item(attribute, options, *args)
    #omg what just happened
    @template.content_tag(:div, class: "control-group") do
      @template.content_tag(:div, class: "controls") do
        options.each do |opt|
          @template.concat(
            label("[#{attribute}][#{opt[1]}]", class: "checkbox") do
              checkbox = @template.check_box_tag("#{@object_name}[#{attribute}][#{opt[1]}]",
                                                       opt[1], 
                                                         (true if 
                                                          @object.public_send(attribute).include?(opt[1].to_s)))

              @template.concat(checkbox)
              @template.concat(opt[0])
            end
          )
        end
      end
    end
  end

  def select_field_item(attribute, options, *args)
    form_input_item(:select, attribute, options, *args)
  end

  def submit_item(title = nil, *args)
    @template.content_tag(:div, class: "control-group") do
      @template.content_tag(:div, class: "controls") do
        submit(title, class: "btn")
      end
    end
  end

  private

  def form_input_item(method, attribute, *args)
    field_item(attribute) do
      @template.public_send(method, 
                            @object_name,
                            attribute, 
                            *args)
    end
  end

  def errors_present?(attribute)
    @object.errors[attribute].any?
  end

end
